@@ -41,13 +41,13 @@ impl So {
 #[test]
 fn linked_list_test1() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
-    let _ = ll.push_front(5, 0)?;
-    let _ = ll.push_front(4, 1)?;
-    let _ = ll.push_front(3, 2)?;
-    let _ = ll.push_front(2, 3)?;
-    let _ = ll.push_front(1, 4)?;
+    let _ = ll.push_front_(5, 0)?;
+    let _ = ll.push_front_(4, 1)?;
+    let _ = ll.push_front_(3, 2)?;
+    let _ = ll.push_front_(2, 3)?;
+    let _ = ll.push_front_(1, 4)?;
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![1_i8, 2, 3, 4, 5]
     );
     Ok(())
@@ -56,13 +56,13 @@ fn linked_list_test1() -> Result<(), MapError> {
 #[test]
 fn linked_list_test2() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
-    let _ = ll.push_front(5, 0)?; // 0
-    let _ = ll.push_front(4, 1)?; // 1
-    let _ = ll.push_front(3, 2)?; // 2
-    let _ = ll.push_front(2, 3)?; // 3
-    let _ = ll.insert_before(3, 6, 4)?; // 4
+    let _ = ll.push_front_(5, 0)?; // 0
+    let _ = ll.push_front_(4, 1)?; // 1
+    let _ = ll.push_front_(3, 2)?; // 2
+    let _ = ll.push_front_(2, 3)?; // 3
+    let _ = ll.insert_before_(3, 6, 4)?; // 4
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![6_i8, 2, 3, 4, 5]
     );
     Ok(())
@@ -71,13 +71,13 @@ fn linked_list_test2() -> Result<(), MapError> {
 #[test]
 fn linked_list_test3() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
-    let _ = ll.push_front(5, 0)?; // 0
-    let _ = ll.push_front(4, 1)?; // 1
-    let _ = ll.push_front(3, 2)?; // 2
-    let _ = ll.push_front(2, 3)?; // 3
-    let _ = ll.insert_before(0, 6, 4)?; // 4
+    let _ = ll.push_front_(5, 0)?; // 0
+    let _ = ll.push_front_(4, 1)?; // 1
+    let _ = ll.push_front_(3, 2)?; // 2
+    let _ = ll.push_front_(2, 3)?; // 3
+    let _ = ll.insert_before_(0, 6, 4)?; // 4
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![2_i8, 3, 4, 6, 5]
     );
     Ok(())
@@ -86,13 +86,13 @@ fn linked_list_test3() -> Result<(), MapError> {
 #[test]
 fn linked_list_test4() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
-    let _ = ll.push_front(5, 0)?; // 0
-    let _ = ll.insert_before(0, 4, 1)?; // 1
-    let _ = ll.insert_before(1, 3, 2)?; // 2
-    let _ = ll.insert_before(2, 2, 3)?; // 3
-    let _ = ll.insert_before(3, 6, 4)?; // 4
+    let _ = ll.push_front_(5, 0)?; // 0
+    let _ = ll.insert_before_(0, 4, 1)?; // 1
+    let _ = ll.insert_before_(1, 3, 2)?; // 2
+    let _ = ll.insert_before_(2, 2, 3)?; // 3
+    let _ = ll.insert_before_(3, 6, 4)?; // 4
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![6_i8, 2, 3, 4, 5]
     );
     Ok(())
@@ -101,23 +101,23 @@ fn linked_list_test4() -> Result<(), MapError> {
 #[test]
 fn linked_list_test5() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
-    let _ = ll.push_front(5, 0)?; // 0
+    let _ = ll.push_front_(5, 0)?; // 0
     let _ = ll.ordered_insert(4, 1, true)?; // 1
     let _ = ll.ordered_insert(3, 2, true)?; // 2
     let _ = ll.ordered_insert(2, 3, true)?; // 3
     let _ = ll.ordered_insert(6, 4, true)?; // 4
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![2_i8, 3, 4, 5, 6]
     );
     ll.clear();
-    let _ = ll.push_front(5, 0)?; // 0
+    let _ = ll.push_front_(5, 0)?; // 0
     let _ = ll.ordered_insert(4, 1, false)?; // 1
     let _ = ll.ordered_insert(3, 2, false)?; // 2
     let _ = ll.ordered_insert(2, 3, false)?; // 3
     let _ = ll.ordered_insert(6, 4, false)?; // 4
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![2_i8, 3, 4, 5, 6]
     );
     Ok(())
@@ -127,10 +127,10 @@ fn linked_list_test5() -> Result<(), MapError> {
 fn linked_list_test6() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
     let _ = ll.ordered_insert(5, 0, true)?; // 0
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![5_i8]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![5_i8]);
     ll.clear();
     let _ = ll.ordered_insert(5, 0, false)?; // 0
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![5_i8]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![5_i8]);
     Ok(())
 }
 
@@ -139,11 +139,11 @@ fn linked_list_test7() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
     let _ = ll.ordered_insert(5, 0, true)?; // 0
     let _ = ll.ordered_insert(5, 0, true)?; // 0
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![5_i8, 5]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![5_i8, 5]);
     ll.clear();
     let _ = ll.ordered_insert(5, 0, false)?; // 0
     let _ = ll.ordered_insert(5, 0, false)?; // 0
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![5_i8, 5]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![5_i8, 5]);
     Ok(())
 }
 
@@ -189,7 +189,7 @@ fn linked_list_test9() -> Result<(), MapError> {
     let _ = ll.ordered_insert(5, 0, true)?;
     let _ = ll.ordered_insert(1, 1, true)?;
     let _ = ll.ordered_insert(2, 2, true)?;
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![1_i8, 2, 5]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![1_i8, 2, 5]);
     ll.clear();
     let _ = ll.ordered_insert(5, 0, false)?; // 0
     let v = ll.pop_front()?;
@@ -203,7 +203,7 @@ fn linked_list_test9() -> Result<(), MapError> {
     let _ = ll.ordered_insert(5, 0, false)?;
     let _ = ll.ordered_insert(1, 1, false)?;
     let _ = ll.ordered_insert(2, 2, false)?;
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![1_i8, 2, 5]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![1_i8, 2, 5]);
     Ok(())
 }
 
@@ -212,10 +212,10 @@ fn linked_list_test10() -> Result<(), MapError> {
     let mut ll = LinkedList::<i8, i8>::default();
     let _ = ll.ordered_insert(1, 0, true)?; // 0
     let _ = ll.ordered_insert(2, 1, false)?; // 1
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![1_i8, 2]);
-    let v = ll.remove(1)?;
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![1_i8, 2]);
+    let v = ll.remove_(1)?;
     assert_eq!(v, Some(2));
-    let v = ll.remove(0)?;
+    let v = ll.remove_(0)?;
     assert_eq!(v, Some(1));
     assert_eq!(ll.len(), 0);
     Ok(())
@@ -227,12 +227,12 @@ fn linked_list_test11() -> Result<(), MapError> {
     let _ = ll.ordered_insert(1, 0, true)?; // 0
     let _ = ll.ordered_insert(2, 1, false)?; // 1
     let _ = ll.ordered_insert(3, 2, true)?; // 2
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![1_i8, 2, 3]);
-    let v = ll.remove(2)?;
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![1_i8, 2, 3]);
+    let v = ll.remove_(2)?;
     assert_eq!(v, Some(3));
-    let v = ll.remove(0)?;
+    let v = ll.remove_(0)?;
     assert_eq!(v, Some(1));
-    let v = ll.remove(1)?;
+    let v = ll.remove_(1)?;
     assert_eq!(v, Some(2));
     assert_eq!(ll.len(), 0);
     Ok(())
@@ -245,12 +245,12 @@ fn linked_list_test12() -> Result<(), MapError> {
     let _ = ll.ordered_insert(1, 0, false)?; // 0
     let _ = ll.ordered_insert(2, 1, true)?; // 1
     let _ = ll.ordered_insert(3, 2, true)?; // 2
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![1_i8, 2, 3]);
-    let v = ll.remove(2)?;
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![1_i8, 2, 3]);
+    let v = ll.remove_(2)?;
     assert_eq!(v, Some(3));
-    let v = ll.remove(0)?;
+    let v = ll.remove_(0)?;
     assert_eq!(v, Some(1));
-    let v = ll.remove(1)?;
+    let v = ll.remove_(1)?;
     assert_eq!(v, Some(2));
 
     let _ = ll.ordered_insert(1, 1, true)?; // 1
@@ -261,33 +261,33 @@ fn linked_list_test12() -> Result<(), MapError> {
     assert_eq!(*ll.get_k(1)?, 1);
     assert_eq!(*ll.get_k(2)?, 3);
 
-    let _ = ll.remove(0)?;
-    let _ = ll.remove(1)?;
-    let _ = ll.remove(2)?;
+    let _ = ll.remove_(0)?;
+    let _ = ll.remove_(1)?;
+    let _ = ll.remove_(2)?;
 
-    let _ = ll.push_front(1, 0)?; // 2
-    let _ = ll.push_front(2, 0)?; // 1
-    let _ = ll.push_front(3, 0)?; // 0
+    let _ = ll.push_front_(1, 0)?; // 2
+    let _ = ll.push_front_(2, 0)?; // 1
+    let _ = ll.push_front_(3, 0)?; // 0
 
     assert_eq!(*ll.get_k(2)?, 1);
     assert_eq!(*ll.get_k(1)?, 2);
     assert_eq!(*ll.get_k(0)?, 3);
 
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![3_i8, 2, 1]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![3_i8, 2, 1]);
 
-    let _ = ll.remove(0)?;
-    let _ = ll.remove(1)?;
-    let _ = ll.remove(2)?;
+    let _ = ll.remove_(0)?;
+    let _ = ll.remove_(1)?;
+    let _ = ll.remove_(2)?;
 
-    let _ = ll.push_back(1, 2)?; // 2
-    let _ = ll.push_back(2, 1)?; // 1
-    let _ = ll.push_back(3, 0)?; // 0
+    let _ = ll.push_back_(1, 2)?; // 2
+    let _ = ll.push_back_(2, 1)?; // 1
+    let _ = ll.push_back_(3, 0)?; // 0
 
     assert_eq!(*ll.get_k(2)?, 1);
     assert_eq!(*ll.get_k(1)?, 2);
     assert_eq!(*ll.get_k(0)?, 3);
 
-    assert_eq!(ll.iter().map(|x| *x).collect::<Vec<_>>(), vec![1_i8, 2, 3]);
+    assert_eq!(ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(), vec![1_i8, 2, 3]);
 
     Ok(())
 }
@@ -296,14 +296,14 @@ fn linked_list_test12() -> Result<(), MapError> {
 #[test]
 fn linked_list_test13() -> Result<(), MapError> {
     let mut ll = LinkedList::<So, i8>::default();
-    let _ = ll.push_front(So::new(3, 0), 0)?; // 0
-    let _ = ll.push_front(So::new(2, 2), 1)?; // 1
-    let _ = ll.push_front(So::new(2, 1), 2)?; // 2
-    let _ = ll.push_front(So::new(1, 3), 3)?; // 3
+    let _ = ll.push_front_(So::new(3, 0), 0)?; // 0
+    let _ = ll.push_front_(So::new(2, 2), 1)?; // 1
+    let _ = ll.push_front_(So::new(2, 1), 2)?; // 2
+    let _ = ll.push_front_(So::new(1, 3), 3)?; // 3
     let _ = ll.ordered_insert(So::new(2, 5), 5, true)?; // 5
 
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![
             So::new(1, 3),
             So::new(2, 5),
@@ -314,7 +314,7 @@ fn linked_list_test13() -> Result<(), MapError> {
     );
     let _ = ll.ordered_insert(So::new(2, 6), 6, true)?; // 6
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![
             So::new(1, 3),
             So::new(2, 6),
@@ -326,7 +326,7 @@ fn linked_list_test13() -> Result<(), MapError> {
     );
     let _ = ll.ordered_insert(So::new(2, 7), 7, false)?; // 7
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![
             So::new(1, 3),
             So::new(2, 6),
@@ -339,7 +339,7 @@ fn linked_list_test13() -> Result<(), MapError> {
     );
     let _ = ll.ordered_insert_pos(So::new(2, 8), 8, 5, false)?; // 7
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![
             So::new(1, 3),
             So::new(2, 6),
@@ -353,7 +353,7 @@ fn linked_list_test13() -> Result<(), MapError> {
     );
     let _ = ll.ordered_insert_pos(So::new(2, 9), 9, 5, true)?; // 7
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![
             So::new(1, 3),
             So::new(2, 9),
@@ -376,14 +376,14 @@ fn linked_list_test14() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, 0, true)?;
     let _ = ll.ordered_insert_pos(So::new(2, 0), 0, 1, true)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(2, 0), So::new(2, 1),]
     );
     ll.clear();
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, 0, false)?;
     let _ = ll.ordered_insert_pos(So::new(2, 0), 0, 1, false)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(2, 1), So::new(2, 0),]
     );
 
@@ -392,7 +392,7 @@ fn linked_list_test14() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, 1, true)?;
     let _ = ll.ordered_insert_pos(So::new(2, 0), 0, 2, true)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(1, 1), So::new(2, 0), So::new(2, 1),]
     );
     ll.clear();
@@ -400,7 +400,7 @@ fn linked_list_test14() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, 1, false)?;
     let _ = ll.ordered_insert_pos(So::new(2, 0), 0, 2, false)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(1, 1), So::new(2, 1), So::new(2, 0),]
     );
     ll.clear();
@@ -409,7 +409,7 @@ fn linked_list_test14() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, 1, false)?;
     let _ = ll.ordered_insert_pos(So::new(2, 0), 0, 1, false)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(1, 1), So::new(2, 1), So::new(2, 0),]
     );
     Ok(())
@@ -423,12 +423,12 @@ fn linked_list_test15() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, 1, false)?;
     let _ = ll.ordered_insert_pos(So::new(2, 2), 1, 0, false)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(2, 0), So::new(2, 1), So::new(2, 2),]
     );
     let _ = ll.ordered_insert_pos(So::new(2, -1), -1, 2, true)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(2, -1), So::new(2, 0), So::new(2, 1), So::new(2, 2),]
     );
     Ok(())
@@ -442,12 +442,12 @@ fn linked_list_test16() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(2, 1), 1, ll.tail(), false)?;
     let _ = ll.ordered_insert_pos(So::new(2, 2), 1, ll.tail(), false)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(2, 0), So::new(2, 1), So::new(2, 2),]
     );
     let _ = ll.ordered_insert_pos(So::new(2, -1), -1, ll.tail(), true)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(2, -1), So::new(2, 0), So::new(2, 1), So::new(2, 2),]
     );
     Ok(())
@@ -462,7 +462,7 @@ fn linked_list_test17() -> Result<(), MapError> {
     let _ = ll.ordered_insert_pos(So::new(4, 4), 0, ll.tail(), false)?;
     let _ = ll.ordered_insert_pos(So::new(2, 2), 1, ll.tail(), false)?;
     assert_eq!(
-        ll.iter().map(|x| x.clone()).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![So::new(1, 0), So::new(2, 1), So::new(2, 2), So::new(4, 4),]
     );
     Ok(())
@@ -476,7 +476,7 @@ fn linked_list_lower_bound_01() -> Result<(), MapError> {
     let _ = ll.ordered_insert(2, 2, false)?; // 2
     let _ = ll.ordered_insert(5, 3, true)?; // 3
     assert_eq!(
-        ll.iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.iter().map(|(x,_)| x.clone()).collect::<Vec<_>>(),
         vec![0_i8, 1, 2, 5]
     );
     // Returns the first element in the container whose key is not considered to go
@@ -520,7 +520,7 @@ fn linked_list_pointer_test01() -> Result<(), MapError> {
     let _ = ll.borrow_mut().ordered_insert(2, 1, false)?; // 1
     let _ = ll.borrow_mut().ordered_insert(3, 2, false)?; // 2
     assert_eq!(
-        ll.borrow().iter().map(|x| *x).collect::<Vec<_>>(),
+        ll.borrow().iter().map(|(x,_)| *x).collect::<Vec<_>>(),
         vec![1_i8, 2, 3]
     );
 
